/*********************************************************************************
* COUNT BITS: This function is used to determine the number of '1' bits in a     *
*   bitboard. Update 2004-01-06: improved with some x86 assembler code           *
*********************************************************************************/
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"





int CntBits (BITBOARD b) {
	#ifdef BCC32
		__asm {
					/* copy the first 4 bytes of the bitboard into ecx for further work */
					mov     ecx, dword ptr b
					/* empty eax */
					xor     eax, eax
					/* if ecx==0, the first 4 bytes seem to be already clear -> continue at read_2nd */
					test    ecx, ecx
					jz      read_2
					
					/* subtract 1 from ecx (the second four bytes) and fold the result into edx */
			count_1:
					lea     edx, [ecx-1]
					/* increase eax by 1 */
					inc     eax
					/* bitwise *and* with ecx and ecx-1 (edx) to remove the MSB */
					and     ecx, edx
					/* loop until ecx becomes zero */
					jnz     count_1
					
					/* copy the second 4 bytes of the bitboard into ecx for further work */
			read_2:
					mov     ecx, dword ptr b+4
					/* if ecx==0, the second 4 bytes seem to be already clear -> finish at l3 */
					test    ecx, ecx
					jz      finish
					
					/* subtract 1 from ecx (the first four bytes) and fold the result into edx */
			count_2:
					lea     edx, [ecx-1]
					/* increase eax by 1 */
					inc     eax
					/* bitwise *and* with ecx and ecx-1 (edx) to remove the MSB */
					and     ecx, edx
					/* loop until ecx becomes zero */
					jnz     count_2
			finish:	
		}
	#else
		int cnt;
		for (cnt=0;b;b>>=1)
			if(b&1) cnt++;
		return cnt;
	#endif
}


/*********************************************************************************
* MOST SIGNIFICANT BIT / LEAST SIGNIFICANT BIT: They determine the first         *
*   respectively the last bit in a bitboard and return its position. It's not    *
*   necessary to optimize them with assembler in that we don't need them often.  *
*********************************************************************************/
int LSB (BITBOARD b) {
	int sq;
	for (sq=0;sq<64 && !(b&mask64[sq]);sq++)
		;
	if(sq>63)
		sq=0; /* this should *not* happen! */
	return sq;
}

int MSB (BITBOARD b) {
	signed int sq;
	for (sq=63;sq<=0 && !(b&mask64[sq]);sq--)
		;
	if(sq<0)
		sq=0; /* this should *not* happen! */
	return sq;
}