
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"



#include <time.h>
#include <sys/time.h>
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"



/*********************************************************************************
* EXPLORE: That's the main search function, based on the techniques MiniMax      *
*   (first described by John Neumann), NegaMax, AlphaBeta and NegaScout.         *
*   It also applies search heuristics, i.e. killer moves, hash moves and static  *
*   move ordering. For further details, refer to the documentation of Philemon.  * 
*********************************************************************************/
int Explore (GAME *g, int alpha, int beta, int depth) { 
	MOVE brmove[MAXMOVES]; /* branch move */
	MOVE hash_bestmove, bestmove;
	SITBACKUP backup;
	int brcount,movenum,value,i;
	int hash_value;
	int hash_type=HASH_UPPERBOUND;
	int hash_result;
	int store_hash=1;
	int brworth[MAXMOVES];
	int key_move, key_worth;
	int legal_moves;
	int original_depth=depth;
	int found_principal_variation=0;

	g->nodes++;
	
	if ((input_query_phase++)>INPUT_QUERY_FREQUENCY) {
		input_query_phase=0;
		DetectInterruption(g);
	}
	if (limit_type==ITERATE_NODE_LIMITED && g->nodes>=limit_value)
		exploration_state=0;
	if (!exploration_state)
		return 0;
	if (DetectDraw(g,&value,0))
		return value;
	
	SwapColors;
	if(TTFetchEntry(g, &hash_value, depth, alpha, beta, &hash_bestmove)!=HASH_UNKNOWN) {
		SwapColors;
		return hash_value;
	}
	SwapColors;

	if(!depth)
		return Quiesce(g, alpha, beta);

	g->ply++;
	SwapColors;
	
	/* calculate possible moves */
	brcount=0;
	brcount=GenerateAP(g, brmove, brcount);
	brcount=GenerateN(g, brmove, brcount);

	/*first loopthrough: give each move a value */
	for (movenum=0; movenum < brcount; movenum++) {
		brworth[movenum]=mCaptured(brmove[movenum])*10;
		if(brmove[movenum]==g->second_killer[g->ply])
			brworth[movenum]=100;
		if(brmove[movenum]==g->first_killer[g->ply])
			brworth[movenum]=200;
		if(mDestination(brmove[movenum])==g->last_destination[g->ply-1])
			brworth[movenum]=250;
		if(brmove[movenum]==hash_bestmove)
			brworth[movenum]=300;
	}
	/* second loopthrough: do an insertion sort with brworth[] as key */
	for(movenum = 1; movenum < brcount; movenum++) {
		key_move = brmove[movenum];
		key_worth = brworth[movenum];
		for(i=movenum-1; i>=0 && brworth[i]<key_worth; i--) {
			/* Move smaller values up one position */
			brmove[i+1] = brmove[i];
			brworth[i+1] = brworth[i];
		}
		/*Insert key into proper position */
		brmove[i+1] = key_move;
		brworth[i+1] = key_worth;
	}

	legal_moves=brcount;
	movenum=0;
	do {
		if(mCaptured(brmove[movenum])==KING) {
			value=MAXINT-g->ply;
			store_hash=0;
		} else {
			/* depth extension if a pawn gets pushed on the 7th line */
			depth=original_depth;
			if(mFigure(brmove[movenum])==PAWN && Rank(mDestination(brmove[movenum]))==ColCmp(6,1))
				depth+=2;
			DoMove(g, brmove[movenum], &backup);
			g->tempmoves[g->ply+1]=brmove[movenum];
			if(Check) {
				value=-MAXINT+g->ply;
				legal_moves--;
			} else {
				/* explore the game tree recursively */
				if(found_principal_variation) {
					value=-Explore(g, -alpha-1, -alpha, depth-1);
					if(value>alpha && value<beta)
						value=-Explore(g, -beta, -alpha, depth-1);
				} else
					value=-Explore(g, -beta, -alpha, depth-1);
			}
			UnDoMove(g, &backup);
		}
		if(value>=beta || !exploration_state) {
			/* BETA CUT: leave to parent node */
			if(exploration_state)
				TTStoreEntry(g, beta, depth, HASH_LOWERBOUND, brmove[movenum]);
			g->ply--;
			SwapColors;
			return beta;
		}
		if(value>alpha) {
			/* ALPHA VALUE: new bestmove found */
			alpha=value;
			hash_type=HASH_EXACT;
			bestmove=brmove[movenum];
			if(g->first_killer[g->ply]!=brmove[movenum]) {
				g->second_killer[g->ply]=g->first_killer[g->ply];
				g->first_killer[g->ply]=brmove[movenum];
			}
			found_principal_variation=1;
		}
		movenum++;
	} while (movenum < brcount);
	
	/* stalemate check */
	if(brcount && !legal_moves && !Check)
		alpha=0;
	if(store_hash && exploration_state)
		TTStoreEntry(g, alpha, depth, hash_type, (hash_type==HASH_EXACT ? bestmove : 0));
	g->ply--;	
	SwapColors;
	return alpha;
}


/*********************************************************************************
* QUIESCENCE SEARCH: This function is being called in Explore() instead of       *
*   Evaluate() after reaching the search horizon. It also uses Alpha-Beta and    *
*   some heuristics.                                                             *
*********************************************************************************/
int Quiesce (GAME *g, int alpha, int beta) { 
	MOVE brmove[MAXMOVES]; /* branch move */
	int brworth[MAXMOVES];
	SITBACKUP backup;
	int brcount,movenum,value,i;
	char buffer[BUFSIZE];
	int hash_type=HASH_UPPERBOUND;
	MOVE hash_bestmove, bestmove;
	int hash_value;
	char store_hash=1;
	int legal_moves;
	char chk;

	g->nodes++;

	if ((input_query_phase++)>INPUT_QUERY_FREQUENCY) {
		input_query_phase=0;
		DetectInterruption(g);
	}
	if (!exploration_state)
		return 0;
	if (limit_type==ITERATE_NODE_LIMITED && g->nodes>=limit_value)
		exploration_state=0;
	if (DetectDraw(g,&value,0))
		return value;
		
	SwapColors;
	if(TTFetchEntry(g, &hash_value, 0, alpha, beta, &hash_bestmove)!=HASH_UNKNOWN) {
		SwapColors;
		return hash_value;
	}
	SwapColors;

	if (g->ply >= MAXPLY-1) {
		Error(errDetails,"maximum depth reached!");
		Dualprintf(" %d ",g->nodes);
		return beta;
	}

	SwapColors;	
	g->ply++;
	
	if (chk=Check) {
		brcount=0;
		brcount=GenerateAP(g, brmove, brcount);
		brcount=GenerateN(g, brmove, brcount);
	} else {
		value=-Evaluate(g,0);
		if(value>=beta || !exploration_state) {
			/* BETA CUT: leave to parent node */
			if(exploration_state)
				TTStoreEntry(g, beta, 0, HASH_LOWERBOUND, 0);
			g->ply--;
			SwapColors;
			return beta;
		}
		if(value>alpha)
			alpha=value;
		
		brcount=0;
		brcount=GenerateAP(g, brmove, brcount);
	}

	legal_moves=brcount;
	if(brcount) {
		/* variables needed for insertion sort */
		int key_move, key_worth;

		/* first loopthrough: give each move a value */
		for (movenum=0; movenum < brcount; movenum++) {
			brworth[movenum]=mCaptured(brmove[movenum])*10;
			if(brmove[movenum]==g->second_killer[g->ply])
				brworth[movenum]=100;
			if(brmove[movenum]==g->first_killer[g->ply])
				brworth[movenum]=200;
			if(mDestination(brmove[movenum])==g->last_destination[g->ply-1])
				brworth[movenum]=250;
			if(brmove[movenum]==hash_bestmove)
				brworth[movenum]=300;
		}
		
		/* second loopthrough: do an insertion sort with brworth[] as key */
		for(movenum = 1; movenum < brcount; movenum++) {
			key_move = brmove[movenum];
			key_worth = brworth[movenum];
			for(i=movenum-1; i>=0 && brworth[i]<key_worth; i--) {
				/* Move smaller values up one position */
				brmove[i+1] = brmove[i];
				brworth[i+1] = brworth[i];
			}
			/*Insert key into proper position */
			brmove[i+1] = key_move;
			brworth[i+1] = key_worth;
		}

		/* third loopthrough: do Quiesce() recursively and hope for as many cutoffs as possible */		
		for (movenum=0; movenum<brcount; movenum++) {
			if(mCaptured(brmove[movenum])==KING) {
				value=MAXINT-g->ply;
				store_hash=0;
			} else {
				DoMove(g, brmove[movenum], &backup);
				g->tempmoves[g->ply+1]=brmove[movenum];
				if(Check) {
					value=-MAXINT+g->ply;
					legal_moves--;
				} else {
					value=-Quiesce(g, -beta, -alpha);
				}
				UnDoMove(g, &backup);
			}
			if(value>=beta || !exploration_state) {
				/* BETA CUT: leave to parent node */
				if(exploration_state)
					TTStoreEntry(g, beta, 0, HASH_LOWERBOUND, brmove[movenum]);
				g->ply--;
				SwapColors;
				return beta;
			}
			if(value>alpha) {
				/* ALPHA VALUE: new bestmove found */
				hash_type=HASH_EXACT;
				alpha=value;
				bestmove=brmove[movenum];
				g->second_killer[g->ply]=g->first_killer[g->ply];
				g->first_killer[g->ply]=brmove[movenum];
			}		
		}
	}
	/* stalemate check */
	if(brcount && !legal_moves && !chk)
		alpha=0;
	if(store_hash && exploration_state)
		TTStoreEntry(g, alpha, 0, hash_type, (hash_type==HASH_EXACT ? bestmove : 0));	
	g->ply--;	
	SwapColors;

	return alpha;
}


/*********************************************************************************
* EXPLORE ROOT: Similar to Explore(), but with some additional code to trace     *
*   back the principal variation and print it out.                               *
*********************************************************************************/
int ExploreRoot (GAME *g, int alpha, int beta, int depth) {
	SITBACKUP backup;
	int movenum,value,i;
	MOVE path[MAXPLY]={0,0,0};
	MOVE bestmove;
	SITBACKUP sb[MAXPLY];
	int path_length;
	int fetch_result;
	int legal_moves;
	int key_move, key_worth;
	int negative_value;

	g->ply=1;

	legal_moves=g->tlcount;
	
	alpha=-MAXINT;

	/* do an insertion sort with g->tlworth[] as key */
	for(movenum = 1; movenum < g->tlcount; movenum++) {
		key_move = g->tlmove[movenum];
		key_worth = g->tlworth[movenum];
		for(i=movenum-1; i>=0 && g->tlworth[i]<key_worth; i--) { 
			/* Move smaller values up one position */
			g->tlmove[i+1] = g->tlmove[i];
			g->tlworth[i+1] = g->tlworth[i];
		}
		/*Insert key into proper position */
		g->tlmove[i+1] = key_move;
		g->tlworth[i+1] = key_worth;
	}

	for(movenum = 0; movenum < g->tlcount; movenum++) {
		path[g->ply]=g->tlmove[movenum]; /* save beginning of the path */
		if(((g->tlmove[movenum]>>15)&7)==KING_CAPTURE) {
			/* king captured -> checkmate? */
			value=MAXINT-g->ply;
		} else {
			DoMove(g, g->tlmove[movenum], &backup);
			SwapColors;
			if(TTFetchEntry(g, &negative_value, depth, 0, 0, &path[g->ply+1])==3) {
				value=-negative_value; /* must be negated since it was stored one ply ahead */
				SwapColors;
			} else {
				SwapColors;
				g->tempmoves[g->ply+1]=g->tlmove[movenum];
				if(Check) {
					value=-MAXINT+g->ply;
					legal_moves--;
				} else
					value=-Explore(g, -beta, -alpha, depth-1); /* explore recursively */
			}

			/* bad variation, therefore we've got no reason to print it out. */
			if (value<=alpha) {	
				UnDoMove(g, &backup);
				/* either continue or abort */
				if (exploration_state) {
					g->tlworth[movenum]=value;
					continue;
				} else {
					if (chess_interface==PHRI)
						Dualprintf("Search ended.");
					return 0;
				}
			}
				
			/*** trace back the principal variation path using the transposition table ***/	
			SwapColors;
			while(TTFetchEntry(g, &nowhere, 0, 0, 0, &path[++g->ply])==HASH_EXACT 
			   && !DetectDraw(g,&value,0)) { /* DetectDraw must be executed AFTER ++g->ply */
				DoMove (g, path[g->ply], &sb[g->ply]);
				SwapColors;
			}
			path_length=--g->ply;
			memset(&path[path_length+1],1,(MAXPLY-path_length-1)*sizeof(MOVE));
			for(; g->ply>1; g->ply--) {
				SwapColors;
				UnDoMove (g, &sb[g->ply]); 
			}
			SwapColors;
			/*** end of backtrace (yeah, took many hours to write these 13 lines) ***/

			UnDoMove(g, &backup);
		}

		g->tlworth[movenum]=value;
		if (!exploration_state) {
			if(chess_interface==PHRI)
				Dualprintf("Search ended.");
			return 0;
		}

		/* output current variation */
		if(chess_interface==PHRI) {
			Dualprintf("->");
			for(i=1;i<=path_length;i++) {
				Dualprintf((ColCmp(i+1,i)%2 ? "  %d..." : "  %d."),ColCmp((i-1)/2,i/2)+g->fullmoves_number);
				if(abs(value)>MATE_THRESHOLD && i==path_length-1) {
					PHRIPrintMove(AddMate(path[i]));
					break;
				}
				PHRIPrintMove(path[i]);
			}
			if(abs(value)<MATE_THRESHOLD)
				Dualprintf(" -> %+.2f\n",(Color ? value : -value)/100.0);
			else
				Dualprintf(" -> #%d\n",path_length/2);
		} else if(chess_interface==UCI && depth>=3) {
			Dualprintf("info currmove ");
			UCIPrintMove(path[1]);
			Dualprintf(" currmovenumber %d depth %d seldepth %d nps %d\n",movenum+1,depth,g->pv_length,NodesPerSecond);
			fflush(stdout);
		}

		/* look for new bestmove */
		if (value>alpha) {
			alpha=value;
			bestmove=g->tlmove[movenum];
			memcpy(g->pv,path,sizeof(MOVE)*MAXPLY);
			g->pv_length=path_length;
		}
	}

	g->best_root_move=bestmove;

	if(!legal_moves && !Check) {
		g->result=RESULT_DRAW_STALE;
		exploration_state=0;
		return alpha=0;
	}

	if(chess_interface==PHRI) {
		Dualprintf("\n\nDepth %d completed, all other moves are inferior.", depth);
		Dualprintf("\n+-------------------+--------------+--------------+---------------------+\n", depth);
		if(abs(alpha)>MATE_THRESHOLD)
			Dualprintf("  Nodes: %-10d | NPS: %-7d | Score: #%-4d | Best move: ", g->nodes, NodesPerSecond, path_length/2);
		else
			Dualprintf("  Nodes: %-10d | NPS: %-7d | Score: %+.2f | Best move: ", g->nodes, NodesPerSecond, (Color ? alpha : -alpha)/100.0);
		PHRIPrintMove(bestmove);
		Dualprintf("\n+-------------------+--------------+--------------+---------------------+\n\n\n", depth);
	} else if(chess_interface==UCI) {
		Dualprintf("info depth %d seldepth %d score cp %d time %d nodes %d nps %d pv ",depth,g->seldepth,alpha,clock()-clock_offset,g->nodes,NodesPerSecond);
		for(i=1;i<=g->pv_length;i++) {
			UCIPrintMove(g->pv[i]);
			Dualprintf(" ");
		}
		Dualprintf("\n");
	}

	return alpha;
}


/*********************************************************************************
* ITERATE: This starts iterative deepening: First, we explore the search tree    *
*   with a depth of one single ply, then with 2 plies and so on... till we       *
*   detect the fulfillment of the the given abort condition.                     *
*********************************************************************************/
int Iterate(GAME *g, char *s) {
	int depth, value, newvalue;
	int alpha, beta;
	int aspiration_phase;
	int col;
	limit_type=ITERATE_NONE;
	
	if (!strncmp(s,"infinite",8))
		limit_type=ITERATE_UNLIMITED;

	if (!strncmp(s,"depth ",6)) {
		s+=6;
		limit_type=ITERATE_DEPTH_LIMITED;
	}	
	if (!strncmp(s,"nodes ",6)) {
		s+=6;
		limit_type=ITERATE_NODE_LIMITED;
	}	
	if (!strncmp(s,"movetime ",9)) {
		s+=9;
		limit_type=ITERATE_TIME_LIMITED;
	}	
	if (!strncmp(s+1,"time ",5)) {
		s+=6;
		limit_type=ITERATE_TIME_PRESSURE;
	}
	
	if (limit_type==ITERATE_NONE) {
		exploration_state=0;
		return 0;
	}
	
	limit_value=atoi(s);
	
	if (limit_type==ITERATE_TIME_PRESSURE) {
		limit_type=ITERATE_TIME_LIMITED;
		limit_value/=40;
	}
	
	clock_offset=clock();
	g->nodes=0;
	exploration_state=1;

	for(col=P_WHITE; col>=P_BLACK; col--)
		g->root_castling_disallowed[col]=g->sit.castling_disallowed[col];
	
	/* calculate all possible moves for CPU */
	g->tlcount=0;
	g->tlcount=GenerateAP(g, g->tlmove, g->tlcount);
	g->tlcount=GenerateN(g, g->tlmove, g->tlcount);
	
	alpha=-MAXINT;
	beta=+MAXINT;
	for(depth=1;(limit_type==ITERATE_DEPTH_LIMITED ? depth<=limit_value : 1);depth++) {
		if(chess_interface==PHRI) {
			Dualprintf("          +-------------------------+  \n");
			Dualprintf("+=========| SEARCHING WITH DEPTH %2.2d |=========+\n",depth);
			Dualprintf("          +-------------------------+  \n");
		}
		aspiration_phase=0;
		repeat_search:
		g->seldepth=0;
		newvalue=ExploreRoot(g,alpha,beta,depth);
		if(depth>=MAXPLY)
			exploration_state=0;
		if(exploration_state) {
			/* aspiration window */
			if(newvalue<=alpha || newvalue>=beta) {
				switch(++aspiration_phase) {
					case 1: alpha=-50; beta=+50; break;
					case 2: alpha=-MAXINT; beta=+MAXINT; break;
				}
				goto repeat_search;
			}
			value=newvalue;
			alpha=value-15;
			beta=value+15;
			/* mate detection */
			if(abs(newvalue)>=MAXINT-2) {
				if(newvalue>0)
					g->result=RESULT_MATED_WHITE;
				else
					g->result=RESULT_MATED_BLACK;
				break;
			}
		} else {
			break;
		}
	}
	if(chess_interface==UCI) {
		Dualprintf("bestmove ");
		UCIPrintMove(g->best_root_move);
		Dualprintf("\n");
	}
	return value;
}