
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"


/*********************************************************************************
* TRANSPOSITION TABLE - STORE ENTRY: We use this function to store several       *
*   features of the current position in a big array, using a hash key.           *
*********************************************************************************/
void TTStoreEntry(GAME *g, int value, int depth, int type, MOVE bestmove) {
	HASHENTRY *this_entry = &transposition_table[g->sit.zobrist % tt_entries];
	int sign=(value<0 ? 1 : 0);
	if (value==0)
		return;
	if (abs(value)>MATE_THRESHOLD)
		depth=0;
	if (abs(value)>MAXINT)
		Error(errDetails,"Hash value overflow!");
	this_entry->verification_key = g->sit.zobrist;
	this_entry->data = ((BITBOARD) bestmove)
	                 | ((BITBOARD) type<<30)
	                 | ((BITBOARD) sign<<32) 
	                 | ((BITBOARD) abs(value)<<33)
	                 | ((BITBOARD) depth<<48);
}


/*********************************************************************************
* TRANSPOSITION TABLE - FETCH ENTRY: Vice versa, we explore the stored hash      *
*   entries to check if there are some useful informations about the current     *
*   position.                                                                    *
*********************************************************************************/
int TTFetchEntry(GAME *g, int *value, int depth, int alpha, int beta, MOVE *bestmove) {
	HASHENTRY *this_entry=&transposition_table[g->sit.zobrist % tt_entries];
	MOVE this_bestmove=(this_entry->data) & mask32[30]-1;
	int this_type=(this_entry->data>>30) & mask32[2]-1;
	int this_value=(this_entry->data>>33) & mask32[15]-1;
	int this_depth=(this_entry->data>>48) & mask32[16]-1;

	if((this_entry->data>>32) & 1) /* signed? */
		this_value=-this_value;
	if (this_entry->verification_key == g->sit.zobrist) {
		*bestmove=this_bestmove;
		if (this_depth >= depth)
			switch(this_type) {
				case HASH_UPPERBOUND:
					if (this_value <= alpha) {
						*value=alpha;
						return HASH_UPPERBOUND;
					}
					break;
				case HASH_LOWERBOUND:
					if (this_value >= beta) {
						*value=beta;
						return HASH_LOWERBOUND;
					}
					break;
				case HASH_EXACT:
					*value=this_value;
					return HASH_EXACT;
			}
	}
	return HASH_UNKNOWN;
}
