
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"


/*********************************************************************************
* ERROR OUTPUT: Nothing very interesting...                                      *
*********************************************************************************/
void Error(char *filename, int line, char *msg, ...) {
	va_list args;
	if(chess_interface==UCI) {
		printf("info string error in %s(%d): ",filename,line,msg);
		fprintf(fp, "info string error in %s(%d): ",filename,line,msg);
	} else {
		printf("Error in %s(%d): ",filename,line,msg);
		fprintf(fp, "Error in %s(%d): ",filename,line,msg);
	}
	va_start(args, msg);
	vprintf(msg, args);
	fflush(stdout);
	vfprintf(fp, msg, args);
	fflush(fp);
	va_end(args);
}


/*********************************************************************************
* DUALPRINTF: This function prints into stdout AND into the log file at the same *
*   time. For easier debugging, almost only this function is used for any output *
*   inside the program.                                                          *
*********************************************************************************/
void Dualprintf(char *format, ...) {
	va_list args;
	va_start(args, format);
	vprintf(format, args);
	fflush(stdout);
	vfprintf(fp, format, args);
	fflush(fp);
	va_end(args);
}


/*********************************************************************************
* READLINE: Platform dependant function that reads from stdin. The standard C    *
*   gets() was just too buggy and didn't cope with the interruption detection.   *
*********************************************************************************/
int ReadLine(char *buffer) {
	int bytes;
	char *eol,*ret;

#ifdef CARBON /* macintosh */
	gets((char *) buffer);
	bytes=strlen(buffer);
	*(buffer+bytes)='\n';
	*(buffer+bytes+1)=0;
#else
	do
		bytes=read(fileno(stdin),buffer,BUFSIZE);
	while (bytes<0 && errno==EINTR);
	*(buffer+bytes)=0;
#endif
	eol=strchr(buffer,'\n');
	*eol=0;
	ret=strchr(buffer,'\r');
	if (ret) *ret=' ';
	return bytes;
}


/*********************************************************************************
* QUERYWIN32INPUT: Here I comply the special ideas from microsoft to do some     *
*   buffered input via pipe or console.                                          *
*********************************************************************************/
#ifdef BCC32
	int QueryWin32Input(void) {
		DWORD win32_bytes_available;
		
		if (win32_pipe) {
			if (!PeekNamedPipe(win32_inputhandle, NULL, 0, NULL, &win32_bytes_available, NULL))
				return 1;
			return win32_bytes_available;
		} else {
			GetNumberOfConsoleInputEvents(win32_inputhandle, &win32_bytes_available);
			return win32_bytes_available <= 1 ? 0 : win32_bytes_available;
		}
	}
#endif


/*********************************************************************************
* DETECTINTERRUPTION: Platform dependant function that checks if something is    *
*   waiting in the input buffer. If yes, ReadLine() is called in the UCI mode    *
*   while we abort the current search if we're in PHRI mode.                     *
*********************************************************************************/
void DetectInterruption(GAME *g) {
#ifdef BCC32
	char buffer[BUFSIZE];
	
	if (chess_interface==UCI && QueryWin32Input()) {
		ReadLine(buffer);
		write(fileno(fp),">>",strlen(">>"));
		write(fileno(fp),buffer,strlen(buffer));
		write(fileno(fp),"(interrupted)\n",strlen("(interrupted)\n"));
		fflush(fp);
		UCIParseCmd(g,buffer);
	}
	if (chess_interface==PHRI && kbhit()) {
		getch();
		exploration_state=0;
	}		
#elif defined(CARBON)
	/* not yet implemented. carbon sucks. */
#endif
	if (limit_type==ITERATE_TIME_LIMITED && (clock()-clock_offset>=limit_value)) {
		exploration_state=0;
		return;
	}
}