
#pragma once


/*********************************************************************************
* MACROS: A lot of functions that were too small or too often used to make a     *
*   conventional function out of it. The big advantage of macros is that they    *
*   do without any function headers and footers, so they'll be directly replaced *
*   by the preprocessor.                                                         *
*********************************************************************************/
/* 
 *  0, 1, 2, 3, 4, 5, 6, 7 
 *  8, 9,10,11,12,13,14,15
 * 16,17,18,19,20,21,22,23 - e.g. 17 = Piece(2,3)
 * 24,25,26,27,28,29,30,31
 * 32,33,34,35,36,37,38,39
 * 40,41,42,43,44,45,46,47
 * 48,49,50,51,52,53,54,55 - e.g. 50 = PieceInverted((3,2)
 * 56,57,58,59,60,61,62,63
 */

/* instead of x%2^y, i'm using the faster x&(2^y-1) everywhere.
 * instead of x/2^y, i'm using the faster x>>y everywhere.
 */


/*********************************************************************************
 * VARIOUS
 *
/* often used macros */
#define Max(a,b) ((a)>(b)?(a):(b))
#define Min(a,b) ((a)>(b)?(b):(a))
#define SqVal(l,r) ((r)*8+(l))
#define SqValInverted(l,r) ((7-(r))*8+(l))
#define Piece(sq) (g->sit.square[sq])
#define PieceInverted(l,r) (g->sit.square[SqValInverted(l,r)])
#define Line(sq) ((sq)&7)
#define Rank(sq) ((sq)>>3)
#define TriCmp(a,b,c) ((a)<(b) && (b)<(c))
#define ColCmp(a,b) (Color ? (a) : (b))
#define BB1 bitboard_1
#define DefMask (mask8[7]) /* used for ParseInput() */
#define LineLiteral(sq) line_char[Line(sq)]
#define RankLiteral(sq) rank_char[Rank(sq)]
#define SetBit(b,sq) b|=mask64[sq]
#define ClearBit(b,sq) b&=imask64[sq]
#define SwapColors { Color=!Color; XORZobristWTM; }            

#define Color (g->sit.color_to_move)
#define Check (AttackedByEnemy(g,g->sit.king[Color]))
#define OwnScore (g->sit.material_score[Color])
#define EnemyScore (g->sit.material_score[!Color])

#define NodesPerSecond (clock()-clock_offset==0 ? 0 : (int) ((g->nodes)/((clock()-clock_offset)/1000.0)))
#define UpdateHalfmoveIndex { g->halfmove_index=2*g->fullmoves_number+!Color-2; }


/* castling */
#define SmallCastlingPossible ( g->sit.king[Color]==ColCmp(E1,E8) && OwnRooks&mask64[ColCmp(H1,H8)] && \
					                      !(castlingfields_king_side[Color]&AllFigures) && \
					                      !(g->sit.castling_disallowed[Color]&1) && !Check &&\
					                      !AttackedByEnemy(g,ColCmp(F1,F8)) && !AttackedByEnemy(g,ColCmp(G1,G8)))
					                      
#define GreatCastlingPossible ( g->sit.king[Color]==ColCmp(E1,E8) && OwnRooks&mask64[ColCmp(A1,A8)] && \
					                      !(castlingfields_queen_side[Color]&AllFigures) && \
					                      !(g->sit.castling_disallowed[Color]&2) && !Check &&\
					                      !AttackedByEnemy(g,ColCmp(D1,D8)) && !AttackedByEnemy(g,ColCmp(C1,C8)))

/***** end of various */


/*********************************************************************************
 * BITBOARDS
 *
/* sets of all figures of a particular side */
#define AllOwn (g->sit.all_figures[Color])
#define AllEnemy (g->sit.all_figures[!Color])
#define AllFigures (g->sit.all_figures[P_BOTH])

/* single piece classes */
#define OwnPawns (g->sit.pawns[Color])
#define OwnKnights (g->sit.knights[Color])
#define OwnBishops (g->sit.bishops[Color])
#define OwnRooks (g->sit.rooks[Color])
#define OwnQueens (g->sit.queens[Color])
#define OwnKing (mask64[g->sit.king[Color]])

#define EnemyPawns (g->sit.pawns[!Color])
#define EnemyKnights (g->sit.knights[!Color])
#define EnemyBishops (g->sit.bishops[!Color])
#define EnemyRooks (g->sit.rooks[!Color])
#define EnemyQueens (g->sit.queens[!Color])
#define EnemyKing (mask64[g->sit.king[!Color]])
/***** end of bitboards */


/*********************************************************************************
 * EVALUATION
 *
/* here are some RelEv (relative evaluation) macros. They depend on the variable 'col'
   in Evaluate() instead of g->sit.color_to_move, like most others do. */
#define EvRelRank(sq) (EvColCmp(Rank(sq),7-Rank(sq)))
#define EvRelFMB(bb) (EvColCmp(MSB(bb),LSB(bb))) /* foremost bit */

#define EvCenterLine (line==D || line==E)
#define EvColCmp(a,b) (col ? (a) : (b))
#define EvCurrentLine (m_line[Line(pos)])
#define EvSide (line>D)
#define EvSideLines ((Line(pos)!=A ? m_line[Line(pos)-1] : 0) | (Line(pos)!=H ? m_line[Line(pos)+1] : 0))
#define EvSideSquares ((Line(pos)!=A ? mask64[pos-1] : 0) | (Line(pos)!=H ? mask64[pos+1] : 0))

#define EvAddPosScore(desc,addend,pos) { pos_score[col]+=(addend); \
                                       if(print_score && (addend)) PHRIStaticEvaluationRow(desc,pos,col,(addend)); }
#define EvHeader(title) { if(print_score) PHRIStaticEvaluationHeader(title); }

#define EvAddSideDanger(desc,addend) { side_danger[col][EvSide]+=(addend); \
                                       if(print_score && (addend)) PHRIDangerousnessRow(desc,EvSide,col,(addend)); }

#define EvOnRefugeLine (EvSide==KINGSIDE ? (line==G||line==H) : (line==A||line==B))
#define EvDisruptRefuge  if (!defective_refuge[col][EvSide] && EvOnRefugeLine) \
						               defective_refuge[col][EvSide]=1

/***** end of evaluation */


/*********************************************************************************
 * MOVE GENERATION
 *
/* state codes */
#define RankState(rank) ((unsigned char) (AllFigures >> ((rank)*8)))

#define LineState(line) ((unsigned char) ((mask64[   line] & AllFigures ? mask8[0] : 0)  \
                                        | (mask64[ 8+line] & AllFigures ? mask8[1] : 0)  \
                                        | (mask64[16+line] & AllFigures ? mask8[2] : 0)  \
                                        | (mask64[24+line] & AllFigures ? mask8[3] : 0)  \
                                        | (mask64[32+line] & AllFigures ? mask8[4] : 0)  \
                                        | (mask64[40+line] & AllFigures ? mask8[5] : 0)  \
                                        | (mask64[48+line] & AllFigures ? mask8[6] : 0)  \
                                        | (mask64[56+line] & AllFigures ? mask8[7] : 0)) )
                               
#define DiagLRState(sq) ((unsigned char) (((diagLR_length[sq]>=1) && (mask64[diagLR_square[0][sq]] & AllFigures) ? mask8[0] : 0)  \
                                        | ((diagLR_length[sq]>=2) && (mask64[diagLR_square[1][sq]] & AllFigures) ? mask8[1] : 0)  \
                                        | ((diagLR_length[sq]>=3) && (mask64[diagLR_square[2][sq]] & AllFigures) ? mask8[2] : 0)  \
                                        | ((diagLR_length[sq]>=4) && (mask64[diagLR_square[3][sq]] & AllFigures) ? mask8[3] : 0)  \
                                        | ((diagLR_length[sq]>=5) && (mask64[diagLR_square[4][sq]] & AllFigures) ? mask8[4] : 0)  \
                                        | ((diagLR_length[sq]>=6) && (mask64[diagLR_square[5][sq]] & AllFigures) ? mask8[5] : 0)  \
                                        | ((diagLR_length[sq]>=7) && (mask64[diagLR_square[6][sq]] & AllFigures) ? mask8[6] : 0)  \
                                        | ((diagLR_length[sq]>=8) && (mask64[diagLR_square[7][sq]] & AllFigures) ? mask8[7] : 0)) )

#define DiagRLState(sq) ((unsigned char) (((diagRL_length[sq]>=1) && (mask64[diagRL_square[0][sq]] & AllFigures) ? mask8[0] : 0)  \
                                        | ((diagRL_length[sq]>=2) && (mask64[diagRL_square[1][sq]] & AllFigures) ? mask8[1] : 0)  \
                                        | ((diagRL_length[sq]>=3) && (mask64[diagRL_square[2][sq]] & AllFigures) ? mask8[2] : 0)  \
                                        | ((diagRL_length[sq]>=4) && (mask64[diagRL_square[3][sq]] & AllFigures) ? mask8[3] : 0)  \
                                        | ((diagRL_length[sq]>=5) && (mask64[diagRL_square[4][sq]] & AllFigures) ? mask8[4] : 0)  \
                                        | ((diagRL_length[sq]>=6) && (mask64[diagRL_square[5][sq]] & AllFigures) ? mask8[5] : 0)  \
                                        | ((diagRL_length[sq]>=7) && (mask64[diagRL_square[6][sq]] & AllFigures) ? mask8[6] : 0)  \
                                        | ((diagRL_length[sq]>=8) && (mask64[diagRL_square[7][sq]] & AllFigures) ? mask8[7] : 0)) )

/* attacks */
#define APawn(sq) (pawn_attacks[sq][Color])
#define AEnemyPawn(sq) (pawn_attacks[sq][!Color])
#define AKnight(sq) (knight_attacks[sq])
#define ABishop(sq) (diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)])
#define ARook(sq) (line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))])
#define AQueen(sq) (diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)] | \
                    line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))])
#define AKing(sq) (king_attacks[sq])

/* non-attacks */
#define NPawn(sq) ((pawn_nonattacks_singlestep[sq][Color] | (Rank(sq)==ColCmp(1,6) && (mask64[sq+ColCmp(8,-8)] & ~AllFigures) ? mask64[sq+ColCmp(16,-16)] : 0)) & ~AllFigures)
#define IsDoubleStep(src,dst) (((Rank(src)==ColCmp(1,6) && (mask64[src+ColCmp(8,-8)] & ~AllFigures) ? mask64[src+ColCmp(16,-16)] : 0) & ~AllFigures) & mask64[dst])
#define NKnight(sq) (knight_attacks[sq] & ~AllFigures)
#define NBishop(sq) ((diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)]) & ~AllFigures)
#define NRook(sq) ((line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))]) & ~AllFigures)
#define NQueen(sq) ((diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)] | \
                     line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))]) & ~AllFigures)
#define NKing(sq) (king_attacks[sq] & ~AllFigures)


/* total: attacks and non-attacks */
#define TPawn(sq) (APawn(sq)&(AllEnemy | (g->sit.enpassant ? mask64[g->sit.enpassant] : 0)) | NPawn(sq))
#define TKnight(sq) (knight_attacks[sq] & ~AllOwn)
#define TBishop(sq) ((diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)]) & ~AllOwn)
#define TRook(sq) ((line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))]) & ~AllOwn)
#define TQueen(sq) ((diagLR_attacks[sq][DiagLRState(sq)] | diagRL_attacks[sq][DiagRLState(sq)] | \
                     line_attacks[sq][LineState(Line(sq))] | rank_attacks[sq][RankState(Rank(sq))]) & ~AllOwn)
#define TKing(sq) ( (king_attacks[sq] | (SmallCastlingPossible ? mask64[ColCmp(G1,G8)] : 0) \
                                      | (GreatCastlingPossible ? mask64[ColCmp(C1,C8)] : 0)) & ~AllOwn)

/***** end of move generation */


/*********************************************************************************
 * XOR MACROS
 */
#define XOROwnFigureset(bitnum) { g->sit.all_figures[Color]^=mask64[bitnum]; g->sit.all_figures[P_BOTH]^=mask64[bitnum]; }
#define XOREnemyFigureset(bitnum) { g->sit.all_figures[!Color]^=mask64[bitnum]; g->sit.all_figures[P_BOTH]^=mask64[bitnum]; }
#define XOROwnSituationBit(figure,bb,bitnum) { g->sit.bb[Color]^=mask64[bitnum]; g->sit.zobrist^=zobrist_figure[Color][figure][bitnum]; XOROwnFigureset(bitnum); }
#define XOREnemySituationBit(figure,bb,bitnum) { g->sit.bb[!Color]^=mask64[bitnum]; g->sit.zobrist^=zobrist_figure[!Color][figure][bitnum]; XOREnemyFigureset(bitnum); }

#define XORZobristOwnKing(sq) (g->sit.zobrist^=zobrist_figure[Color][KING][sq])
#define XORZobristEP(sq) (g->sit.zobrist^=zobrist_enpassant[sq])
#define XORZobristCastling (g->sit.zobrist^=zobrist_castling_disallowed[Color][g->sit.castling_disallowed[Color]])
#define XORZobristWTM (g->sit.zobrist^=zobrist_whitetomove)


/*********************************************************************************
 * MACROS TO EDIT OR DISPLAY MOVE VARIABLES 
 */
#define mMate(move)        ((move)>>23)
#define mCaptured(move)    ((move>>20)&7)
#define mPromotion(move)   ((move>>18)&3)
#define mMoveType(move)    ((move>>15)&7)
#define mDestination(move) ((move>>9)&63)
#define mSource(move)      ((move>>3)&63)
#define mFigure(move)      ((move)&7)
#define AddMate(move)      ((move)|mask32[23])
#define UCIPrintMove(mv)   ( Dualprintf("%c%c%c%c",LineLiteral((mv>>3)&63), RankLiteral((mv>>3)&63), LineLiteral((mv>>9)&63), RankLiteral((mv>>9)&63)))

/*********************************************************************************
 * DEBUG MACROS
 */
#define dCreateMove(figure,src,dst,move_type,promotion,captured) (((captured)<<20)|(promotion<<18)|(move_type<<15)|(dst<<9)|(src<<3)|figure)
#define errDetails __FILE__,__LINE__
