/*********************************************************************************
*                                                                                *
* "CHESS ENGINE IN C"                                                            *
*                                                                                *
* "Maturittsarbeit" 2003/2004 (there is no english counterpart to this..)       *
* programming: Dominik Kser, C5a/C6a, KZO Wetzikon                              *
* coaching: Oliver Seipel, Spl, KZO Wetzikon                                     *
*                                                                                * 
* Log:                                                                           *
*                                                                                * 
* xmas holidays 2002/03: working me through 'C/C++ Kompendium' by Dirk Louis     *
* beginning of january 2003: starting reading 'Algorithms in C' by O'Reilly      *
*                                                                                * 
* 17.1.2003, 12.40: once it happened a friday afternoon on an apple station at   *
*  the KZO: the project "helloworld_chess_asdf1" was created.                    *
*                                                                                *
* 1.2.2003, 3.20: yeah, the visual representation of the chessboard is working   *
*  now in cute ASCII-chars, and the inputParse() function doesn't return anymore *
*  only adressing errors.                                                        *
*                                                                                *
* 2.2.2003, 1.10: i welcome the SITUATION-struct and the bitboards here.         *
*                                                                                *
* 10.2.2003, 16.00: i added are a lot of new features, like all the bitboard     *
*  management functions and the brand new reviewMove(). Now each move in the     *
*  algebraic and the UCI notation should be recognized. Should.                  *
*                                                                                *
* 16.3.2003, 21.37: after a break of about a month due to a local music homepage,*
*  i just could complete the rank/line attack board initialisation routines.     *
*                                                                                *
* to be continued...                                                             *
*                                                                                *
*   ...sometime...                                                               *
*                                                                                *
*      ...erm...                                                                 *
*                                                                                *
*         ...hopefully...                                                        *
*                                                                                *
*********************************************************************************/

/* BCC32, GCC, CARBON */
//#define BCC32

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"



#include <time.h>
#include <errno.h>
#include <unistd.h>


//#include <io.h>
#if defined(BCC32)
#	include <windows.h>
# include <conio.h>
#elif defined(CARBON)
#	include <unistd.h>
#	include <unix.h>
#	include <Events.h>
#endif

#if defined(BCC32)
	/* BCB 5 warning W8060: possible unwanted assignment */
	#pragma warn -8060
#elif defined(CARBON)
	#pragma warn_possunwant off
#else
	#pragma warn -pia
#endif


/*********************************************************************************
* HEADER FILES: DECLARATIONS                                                     *
*********************************************************************************/
#include "const.h"
#include "macros.h"
#include "types.h"
#include "proto.h"

#include "globals.h"
#include "matrices.h"


/*********************************************************************************
* OTHER SOURCE FILES                                                             *
*********************************************************************************/
#include "bitops.c"
#include "debug.c"
#include "domove.c"
#include "evaluate.c"
#include "explore.c"
#include "generate.c"
#include "hash.c"
#include "init.c"
#include "io.c"
#include "parse.c"
#include "phri.c"
#include "tool.c"
#include "uci.c"
#include "undomove.c"


/*********************************************************************************
* MAIN		                                                                       *
*********************************************************************************/
int main(void) {
	GAME *g;
	MOVE bestmove;
	char buffer[BUFSIZE]={0};
	int value,bestvalue,movenum,i,j,chk,col;

	srand((unsigned int) time(NULL));
	g=(GAME*) malloc(sizeof(GAME));
	Init(g);
	
	/*** various I/O tasks */
	setbuf(stdout, NULL);
	fp=fopen("main.out","w");

	Dualprintf("Philemon Chess Engine(JA)\nPHRI Interface initialized.\n\n");
	
	/***** To start with black, use the following code:*/
	 Color=0; XORZobristWTM;
	 
	
	do {
		/* used for Evaluate() with print_score==1 */
		for(col=P_WHITE; col>=P_BLACK; col--)
			g->root_castling_disallowed[col]=g->sit.castling_disallowed[col];
		
		if(chess_interface==PHRI) {
			PHRIDisplayBoard(g);
			/* player in check? */
			if(Check) { 
				Dualprintf("      +===================+\n");
				Dualprintf("      | ! ! C H E C K ! ! |\n");
				Dualprintf("      +===================+\n");
			}
			Dualprintf("\n");
			Dualprintf("\n%d%s> ",g->fullmoves_number,(Color ? ". (white)" : "... (black)"));
		}

		ReadLine(buffer);
		write(fileno(fp),">>",strlen(">>"));
		write(fileno(fp),buffer,strlen(buffer));
		write(fileno(fp),"\n",strlen("\n"));
		fflush(fp);

		if (ParseInterfaceChange(buffer))
			continue;
		
		if (chess_interface==UCI) {
			strcpy(cmd_queue,buffer);
			do
				UCIParseCmd(g,cmd_queue);
			while(*cmd_queue);
		}

		if (chess_interface==PHRI)
			if (g->prec_move=PHRIParseCmd(g,buffer)) {			

				/* first, do the human move */
				DoMove(g, g->prec_move, &g->passed_backup[2*g->fullmoves_number+!Color-2]);
				if(Check) {
					Dualprintf("illegal move, you're in check!");
					UnDoMove(g, &g->passed_backup[2*g->fullmoves_number+!Color-2]);
					continue;
				}
				
				/* draw or mate after human move? */
				DetectDraw(g,&nowhere,1);
				if(DetectEndOfGame(g)) continue;
				
				/* update movecounter & swap colors */
				if(Color==P_BLACK) g->fullmoves_number++;
				SwapColors;
				UpdateHalfmoveIndex;
				
				/* start iterative deepening until the pre-defined level is reached */
				strcpy(buffer,"depth ");
				*(buffer+strlen(buffer))='0'+phri_level;
				*(buffer+strlen(buffer)+1)='\0';
				Iterate(g,buffer);
				
				/* iterate has possibly changed g->result */
				if(DetectEndOfGame(g)) continue;
				
				/* do the computer's move */
				DoMove(g, g->best_root_move, &g->passed_backup[2*g->fullmoves_number+!Color-2]);
				
				/* draw or mate after computer's move? */
				DetectDraw(g,&nowhere,1);
				if(DetectEndOfGame(g)) continue;
				
				/* update movecounter & swap colors */
				if(Color==P_BLACK) g->fullmoves_number++;
				SwapColors;
				UpdateHalfmoveIndex;
				
				Dualprintf("      |--> My move: ");
				PHRIPrintMove(g->best_root_move);
				Dualprintf("\n");
			}
	} while (program_state);	
	
	fprintf(fp,"--- normal process shutdown ---");
	fclose(fp);
	return 0;
}
