
#pragma once

/*********************************************************************************
* MATRICES                                                                       *
*********************************************************************************/

/* the initial position on the chess board */
static const signed char initial_sq[64] = { 4, 2, 3, 5, 6, 3, 2, 4,
                                     1, 1, 1, 1, 1, 1, 1, 1,
                                     0, 0, 0, 0, 0, 0, 0, 0,
                                     0, 0, 0, 0, 0, 0, 0, 0,
                                     0, 0, 0, 0, 0, 0, 0, 0,
                                     0, 0, 0, 0, 0, 0, 0, 0,
                                    -1,-1,-1,-1,-1,-1,-1,-1,
                                    -4,-2,-3,-5,-6,-3,-2,-4 };

/* the whole chessboard, rotated by 90 degrees */
static const int rot90[64] = {56,48,40,32,24,16, 8, 0,
                       57,49,41,33,25,17, 9, 1,
                       58,50,42,34,26,18,10, 2,
                       59,51,43,35,27,19,11, 3,
                       60,52,44,36,28,20,12, 4,
                       61,53,45,37,29,21,13, 5,
                       62,54,46,38,30,22,14, 6,
                       63,55,47,39,31,23,15, 7};

/* used to extract the current rank state. see <init.c> for details */
static const int slide_shift[64] = {0, 0, 0, 0, 0, 0, 0, 0,
                             8, 8, 8, 8, 8, 8, 8, 8,
                            16,16,16,16,16,16,16,16,
                            24,24,24,24,24,24,24,24,
                            32,32,32,32,32,32,32,32,
                            40,40,40,40,40,40,40,40,
                            48,48,48,48,48,48,48,48,
                            56,56,56,56,56,56,56,56};

/* diagonal length for diagonals from left to right */
static const int diagLR_length[64] = {8, 7, 6, 5, 4, 3, 2, 1,
                               7, 8, 7, 6, 5, 4, 3, 2,
                               6, 7, 8, 7, 6, 5, 4, 3,
                               5, 6, 7, 8, 7, 6, 5, 4,
                               4, 5, 6, 7, 8, 7, 6, 5,
                               3, 4, 5, 6, 7, 8, 7, 6,
                               2, 3, 4, 5, 6, 7, 8, 7,
                               1, 2, 3, 4, 5, 6, 7, 8};

/* diagonal length for diagonals from right to left */
static const int diagRL_length[64] = {1, 2, 3, 4, 5, 6, 7, 8,
                               2, 3, 4, 5, 6, 7, 8, 7,
                               3, 4, 5, 6, 7, 8, 7, 6,
                               4, 5, 6, 7, 8, 7, 6, 5,
                               5, 6, 7, 8, 7, 6, 5, 4,
                               6, 7, 8, 7, 6, 5, 4, 3,
                               7, 8, 7, 6, 5, 4, 3, 2,
                               8, 7, 6, 5, 4, 3, 2, 1};

                               
/* just because UltraEdit32 shows me numbers in red and letters in black.  * 
 * this way it becomes more clear for the following matrices.              */
#define XX 0 


/* half of the fields are unused. fortunately, we've got plenty of memory. */
static const int diagLR_square[8][64] = { {0, 1, 2, 3, 4, 5, 6, 7,
                                    8, 0, 1, 2, 3, 4, 5, 6,
                                   16, 8, 0, 1, 2, 3, 4, 5,
                                   24,16, 8, 0, 1, 2, 3, 4,
                                   32,24,16, 8, 0, 1, 2, 3,
                                   40,32,24,16, 8, 0, 1, 2,
                                   48,40,32,24,16, 8, 0, 1,
                                   56,48,40,32,24,16, 8, 0},

                                   {9,10,11,12,13,14,15,XX,
                                   17, 9,10,11,12,13,14,15,
                                   25,17, 9,10,11,12,13,14,
                                   33,25,17, 9,10,11,12,13,
                                   41,33,25,17, 9,10,11,12,
                                   49,41,33,25,17, 9,10,11,
                                   57,49,41,33,25,17, 9,10,
                                   XX,57,49,41,33,25,17, 9},

                                  {18,19,20,21,22,23,XX,XX,
                                   26,18,19,20,21,22,23,XX,
                                   34,26,18,19,20,21,22,23,
                                   42,34,26,18,19,20,21,22,
                                   50,42,34,26,18,19,20,21,
                                   58,50,42,34,26,18,19,20,
                                   XX,58,50,42,34,26,18,19,
                                   XX,XX,58,50,42,34,26,18},

                                  {27,28,29,30,31,XX,XX,XX,
                                   35,27,28,29,30,31,XX,XX,
                                   43,35,27,28,29,30,31,XX,
                                   51,43,35,27,28,29,30,31,
                                   59,51,43,35,27,28,29,30,
                                   XX,59,51,43,35,27,28,29,
                                   XX,XX,59,51,43,35,27,28,
                                   XX,XX,XX,59,51,43,35,27},

                                  {36,37,38,39,XX,XX,XX,XX,
                                   44,36,37,38,39,XX,XX,XX,
                                   52,44,36,37,38,39,XX,XX,
                                   60,52,44,36,37,38,39,XX,
                                   XX,60,52,44,36,37,38,39,
                                   XX,XX,60,52,44,36,37,38,
                                   XX,XX,XX,60,52,44,36,37,
                                   XX,XX,XX,XX,60,52,44,36},

                                  {45,46,47,XX,XX,XX,XX,XX,
                                   53,45,46,47,XX,XX,XX,XX,
                                   61,53,45,46,47,XX,XX,XX,
                                   XX,61,53,45,46,47,XX,XX,
                                   XX,XX,61,53,45,46,47,XX,
                                   XX,XX,XX,61,53,45,46,47,
                                   XX,XX,XX,XX,61,53,45,46,
                                   XX,XX,XX,XX,XX,61,53,45},

                                  {54,55,XX,XX,XX,XX,XX,XX,
                                   62,54,55,XX,XX,XX,XX,XX,
                                   XX,62,54,55,XX,XX,XX,XX,
                                   XX,XX,62,54,55,XX,XX,XX,
                                   XX,XX,XX,62,54,55,XX,XX,
                                   XX,XX,XX,XX,62,54,55,XX,
                                   XX,XX,XX,XX,XX,62,54,55,
                                   XX,XX,XX,XX,XX,XX,62,54},

                                  {63,XX,XX,XX,XX,XX,XX,XX,
                                   XX,63,XX,XX,XX,XX,XX,XX,
                                   XX,XX,63,XX,XX,XX,XX,XX,
                                   XX,XX,XX,63,XX,XX,XX,XX,
                                   XX,XX,XX,XX,63,XX,XX,XX,
                                   XX,XX,XX,XX,XX,63,XX,XX,
                                   XX,XX,XX,XX,XX,XX,63,XX,
                                   XX,XX,XX,XX,XX,XX,XX,63} };            


static const int diagRL_square[8][64] = { {0, 1, 2, 3, 4, 5, 6, 7,
                                    1, 2, 3, 4, 5, 6, 7,15,
                                    2, 3, 4, 5, 6, 7,15,23,
                                    3, 4, 5, 6, 7,15,23,31,
                                    4, 5, 6, 7,15,23,31,39,
                                    5, 6, 7,15,23,31,39,47,
                                    6, 7,15,23,31,39,47,55,
                                    7,15,23,31,39,47,55,63},

                                  {XX, 8, 9,10,11,12,13,14,
                                    8, 9,10,11,12,13,14,22,
                                    9,10,11,12,13,14,22,30,
                                   10,11,12,13,14,22,30,38,
                                   11,12,13,14,22,30,38,46,
                                   12,13,14,22,30,38,46,54,
                                   13,14,22,30,38,46,54,26,
                                   14,22,30,38,46,54,62,XX},

                                  {XX,XX,16,17,18,19,20,21,
                                   XX,16,17,18,19,20,21,29,
                                   16,17,18,19,20,21,29,37,
                                   17,18,19,20,21,29,37,45,
                                   18,19,20,21,29,37,45,53,
                                   19,20,21,29,37,45,53,61,
                                   20,21,29,37,45,53,61,XX,
                                   21,29,37,45,53,61,XX,XX},

                                  {XX,XX,XX,24,25,26,27,28,
                                   XX,XX,24,25,26,27,28,36,
                                   XX,24,25,26,27,28,36,44,
                                   24,25,26,27,28,36,44,52,
                                   25,26,27,28,36,44,52,60,
                                   26,27,28,36,44,52,60,XX,
                                   27,28,36,44,52,60,XX,XX,
                                   28,36,44,52,60,XX,XX,XX},

                                  {XX,XX,XX,XX,32,33,34,35,
                                   XX,XX,XX,32,33,34,35,43,
                                   XX,XX,32,33,34,35,43,51,
                                   XX,32,33,34,35,43,51,59,
                                   32,33,34,35,43,51,59,XX,
                                   33,34,35,43,51,59,XX,XX,
                                   34,35,43,51,59,XX,XX,XX,
                                   35,43,51,59,XX,XX,XX,XX},

                                  {XX,XX,XX,XX,XX,40,41,42,
                                   XX,XX,XX,XX,40,41,42,50,
                                   XX,XX,XX,40,41,42,50,58,
                                   XX,XX,40,41,42,50,58,XX,
                                   XX,40,41,42,50,58,XX,XX,
                                   40,41,42,50,58,XX,XX,XX,
                                   41,42,50,58,XX,XX,XX,XX,
                                   42,50,58,XX,XX,XX,XX,XX},

                                  {XX,XX,XX,XX,XX,XX,48,49,
                                   XX,XX,XX,XX,XX,48,49,57,
                                   XX,XX,XX,XX,48,49,57,XX,
                                   XX,XX,XX,48,49,57,XX,XX,
                                   XX,XX,48,49,57,XX,XX,XX,
                                   XX,48,49,57,XX,XX,XX,XX,
                                   48,49,57,XX,XX,XX,XX,XX,
                                   49,57,XX,XX,XX,XX,XX,XX},

                                  {XX,XX,XX,XX,XX,XX,XX,56,
                                   XX,XX,XX,XX,XX,XX,56,XX,
                                   XX,XX,XX,XX,XX,56,XX,XX,
                                   XX,XX,XX,XX,56,XX,XX,XX,
                                   XX,XX,XX,56,XX,XX,XX,XX,
                                   XX,XX,56,XX,XX,XX,XX,XX,
                                   XX,56,XX,XX,XX,XX,XX,XX,
                                   56,XX,XX,XX,XX,XX,XX,XX} };


static const int empty_board[64] = {0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0};




/* ATTENTION: these arrays are only written for black pieces since this seems more
              'logical' for a chessplayer. it's like looking at the board from white's
              point of view. (A1 = down left, etc.). we let the computer fill in the values 
              for white in init.c.
              
              well, I called these arrays sailingboat-arrays just because they looked 
              at me like ships with an 8x8 sail in the middle. :)
              
              empty_sailingboat[][] is only used to create new boats via copy&paste.
              */
               

static signed char empty_sailingboat[2][64] = { {  0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0},                                       
                         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                          0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };


               
static signed char outpost_value[2][64] = { {  0,  0,  0,  0,  0,  0,  0,  0,
                                        0,  0,  0,  0,  0,  0,  0,  0,
                                        0,  0,  8, 12, 12,  8,  0,  0,
                                        0,  2, 12, 16, 16, 12,  2,  0,
                                        0,  2, 12, 20, 20, 12,  2,  0,
                                        0,  0,  0,  0,  0,  0,  0,  0,  /* 3rd rank results */
                                        0,  0,  0,  0,  0,  0,  0,  0,  /* in no bonus due to */
                                        0,  0,  0,  0,  0,  0,  0,  0}, /* e.g. 1. Sg1-f3 */
                     {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };


static signed char knight_posvalue[2][64] = { {-16,-16, -8, -8, -8, -8,-16,-16,
                                        -16,-16,  4,  4,  4,  4,-16,-16,
                                         -8,  2,  6,  6,  6,  6,  2, -8,
                                         -8,  2,  6,  6,  6,  6,  2, -8,
                                         -8,  2,  4,  4,  4,  4,  2, -8,
                                         -8,  2,  2,  2,  2,  2,  2, -8,
                                         -8, -8,  0,  0,  0,  0, -8, -8,
                                        -16, -8, -8, -8, -8, -8, -8,-16},                                       
                       {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };


static signed char bishop_posvalue[2][64] = { {-16,-16,  0,  0,  0,  0,-16,-16,
                                        -16,  2,  4,  4,  4,  4,  2,-16,
                                         -8,  2,  6,  6,  6,  6,  2, -4,
                                         -8,  2,  6,  6,  6,  6,  2, -4,
                                         -8,  2,  4,  4,  4,  4,  2, -4,
                                         -8,  2,  2, -4, -1,  2,  2, -4, /* often blocked d3/d6 in */
                                        -16,  2,  0,  0,  0,  0,  2,-16, /* earlier versions -> -4 */
                                        -16,-16, -8, -8, -8, -8,-16,-16},                                       
                       {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };
                                             

static signed char rook_posvalue[2][64] = { {  0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0,
                                        0,  0,  4,  6,  6,  4,  0,  0},                                       
                     {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };


static signed char queen_posvalue[2][64] = { {  0,  0,  0,  0,  0,  0,  0,  0,
                                         0,  0,  4,  5,  5,  4,  0,  0,
                                         0,  2,  4, 10, 10,  4,  2,  0,
                                         0,  2, 10, 12, 12, 10,  2,  0,
                                       -10,  2, 10, 12, 12, 10,  2,-10,
                                       -10,-10,  4, 10, 10,  4,-10,-10,
                                       -10,  2,  8,  8,  8,  8,  2,-10,
                                       -10, -8,  0,  0,  0,  0, -8,-10},                                       
                         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                          0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };                        


static signed char pawn_posvalue[2][64] = { {  0,  0,  0,  0,  0,  0,  0,  0,
                                       28, 28, 35, 42, 45, 35, 28, 28,
                                       -9, -3,  7, 12, 15,  7, -3, -9,
                                      -10,-10,  6,  9, 10,  6,-11,-10,
                                      -11,-11,  4,  5,  6,  2,-11,-11,
                                      -11,-11,  0,  0,  1,  0,-11,-11,
                                       -6, -6,  4,  5,  5,  4, -6, -6,
                                        0,  0,  0,  0,  0,  0,  0,  0},                                       
                     {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };
                            

static signed char king_mid_posvalue[2][64] = { {  0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,  0,  0,  0,  0,  0,
                                            0,  0,  0,-10,-15,-10,  0,  0,
                                            5, 10, 18, -8, -3, -8, 23, 10},                                       
                         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                          0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} };
         

