
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"


/*********************************************************************************
* PARSE PHILEMON HUMAN READABLE INTERFACE (PHRI)                                 *
*********************************************************************************/
int PHRIParseCmd(GAME *g, char *buffer) {
	char *s;
	int tmp;
	
	/* single-literal commands */
	if (buffer[0]=='.') {
		switch(buffer[1]) {
			case 'h': Iterate(g,"infinite"); break;
			case 'e': Evaluate(g,1); break;
			case 'q': program_state=0; break;
			case 'l': if(TriCmp(0,tmp=buffer[2]-'0',10))
				Dualprintf("New search depth: %d",tmp);
				phri_level=tmp;
			break;
			case 'u':
				if(g->fullmoves_number<=1) return 0;
				SwapColors;
				if(Color==P_BLACK) g->fullmoves_number--;
				UpdateHalfmoveIndex;
				UnDoMove(g, &g->passed_backup[2*g->fullmoves_number+!Color-2]);
				SwapColors;
				if(Color==P_BLACK) g->fullmoves_number--;
				UpdateHalfmoveIndex;
				UnDoMove(g, &g->passed_backup[2*g->fullmoves_number+!Color-2]);
			break;
		}
		return 0;
	}
	return ParseMove(g,buffer);
}


/*********************************************************************************
* PHRI: PRINT THE HEADER OF THE EVALUATION TABLE                                 *
*********************************************************************************/
void PHRIStaticEvaluationHeader(char *title) {
	Dualprintf("          %20.20s -----------+-COLOR-+--SCORE-+---\n",title);
}


/*********************************************************************************
* PHRI: PRINT A NEW ROW IN THE EVALUATION TABLE (in centipawns)                  *
*********************************************************************************/
void PHRIStaticEvaluationRow(char *description, int pos, int col, int value) {
	char pos_literal[5]="....";
	if(pos>-1) {
		pos_literal[0]='(';
		pos_literal[1]=LineLiteral(pos);
		pos_literal[2]=RankLiteral(pos);
		pos_literal[3]=')';
	}
	Dualprintf("  %34.34s %4.4s | %5.5s | %+6.2f |\n",description,pos_literal,
	           (col<P_BOTH ? (col==P_WHITE ? "white" : "black") : "both"),
	           (col ? value : -value)/100.0);
}


/*********************************************************************************
* PHRI: PRINT A NEW ROW IN THE EVALUATION TABLE (in king defects)                *
*********************************************************************************/
void PHRIDangerousnessRow(char *description, int side, int col, int value) {
	Dualprintf("  %34.34s [%cS] | %5.5s | %3d kd |\n", description, (side ? 'K' : 'Q'),
	           (col<P_BOTH ? (col==P_WHITE ? "white" : "black") : "both"), value);
}


/*********************************************************************************
* PHRI: PRINT ANY GIVEN MOVE IN LONG ALGEBRAIC NOTATION                          *
*********************************************************************************/
void PHRIPrintMove(MOVE mv) {
	if(mMoveType(mv)==CASTLING) {
		if(Line(mDestination(mv))==G)
			Dualprintf("0-0");
		else if(Line(mDestination(mv))==C)
			Dualprintf("0-0-0");
		else
			Error(errDetails,"Illegal Castling in PHRIPrintMove() detected!");
	} else
		Dualprintf("%c%c%c%c%c%c",pieceSymbol[(mv)&7], LineLiteral((mv>>3)&63), RankLiteral((mv>>3)&63), (mCaptured(mv) ? 'x' : '-'),LineLiteral((mv>>9)&63), RankLiteral((mv>>9)&63));
	if(mMoveType(mv)==ENPASSANT_CAPTURE)
		Dualprintf(" e.p.");
	if(mMoveType(mv)==PROMOTION)
		Dualprintf("=%c",pieceSymbol[mPromotion(mv)+2]);
	if(mMate(mv))
		Dualprintf("#");
}


/*********************************************************************************
* DISPLAYBOARD                                                                   *
*********************************************************************************/
void PHRIDisplayBoard(GAME *g) {
	int x,y;
	char currentSquare[4];
	
	Dualprintf("\n");
	for(y=0; y<8; y++) {
		Dualprintf("+---+---+---+---+---+---+---+---+\n");
		for (x=0; x<8; x++) {
			strcpy(currentSquare,"   ");
			if (PieceInverted(x,y)<0) strcpy(currentSquare, "( )");
			if(TriCmp(0,abs(PieceInverted(x,y)),KING+1))
				currentSquare[1]=pieceSymbol[abs(PieceInverted(x,y))];
			Dualprintf("|%s",currentSquare);
		}
		Dualprintf("| %d\n",8-y);
	}
	Dualprintf("+---+---+---+---+---+---+---+---+\n");
	Dualprintf("  a   b   c   d   e   f   g   h  \n");
	Dualprintf("\n");
}