


#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"

/*********************************************************************************
* IDENTIFY THE UCI ENGINE: This routine is called after turning the engine in    *
*   the UCI mode via 'uci' to identify itself.                                   *
*********************************************************************************/
void UCIIdentify() {
	Dualprintf("\n");
	Dualprintf("id name Philemon C (JA)\n");
	Dualprintf("id author Dominik Kser\n");
	Dualprintf("option name Hash type spin default 32 min 8 max 512\n");
	Dualprintf("option name FirstAspirationWindow type spin default %d min 0 max 250\n",first_aspiration_window);
	Dualprintf("option name SecondAspirationWindow type spin default %d min 0 max 250\n",second_aspiration_window);
	Dualprintf("option name KingSafetyCare1 type spin default %d min 0 max 200\n",king_safety_care_1st);
	Dualprintf("option name KingSafetyCare2 type spin default %d min 0 max 200\n",king_safety_care_2nd);
	Dualprintf("option name KingSafetyUpperLimit type spin default %d min 0 max 1500\n",king_safety_upper_limit);
	Dualprintf("option name Clear Hash type button\n");
	Dualprintf("uciok\n");
	fflush(stdout);
}


/*********************************************************************************
* PARSE THE UNIVERSAL CHESS INTERFACE (UCI) COMMANDS                             *
*********************************************************************************/
int UCIParseCmd(GAME *g, char *str) {
	SITBACKUP backup;
	MOVE move;
	char buffer[BUFSIZE];
	char *s, *tok;

	memset(buffer,0,sizeof(buffer));
	strcpy(buffer,str);
	*cmd_queue=0;	
		
	/* UCI: 'isready' */
	if (!strcmp(buffer,"isready")) {
		Dualprintf("readyok\n");
		return 0;
	}

	/* UCI: 'stop' */
	if (!strcmp(buffer,"stop")) {
		exploration_state=0;
		return 0;
	}

	/* UCI: 'quit' */
	if (!strcmp(buffer,"quit"))
		if (exploration_state) {
			exploration_state=0;
			strcpy(cmd_queue,buffer);
		} else {
			program_state=0;
			return 0;
		}
	
	/* UCI: 'setoption' */
	if (!strncmp(buffer,"setoption name ",15))
		{
			s=buffer+15;
	 		if (!strncmp(s,"Clear Hash",10)) {
				memset(transposition_table,0,(long) hash_megabytes*1048576);
			}	
			if (!strncmp(s,"Hash value ",11)) {
				s+=11;
				hash_megabytes=atoi(s);
				InitTranspositionTable(hash_megabytes);
			}
			if (!strncmp(s,"KingSafetyCare1 value ",22)) {
				s+=22;
				king_safety_care_1st=atoi(s);
			}
			if (!strncmp(s,"KingSafetyCare2 value ",22)) {
				s+=22;
				king_safety_care_2nd=atoi(s);
			}
			if (!strncmp(s,"FirstAspirationWindow value ",28)) {
				s+=28;
				first_aspiration_window=atoi(s);
			}
			if (!strncmp(s,"SecondAspirationWindow value ",29)) {
				s+=29;
				second_aspiration_window=atoi(s);
			}
			if (!strncmp(s,"KingSafetyUpperLimit value ",27)) {
				s+=27;
				king_safety_upper_limit=atoi(s);
			}
		}

	/* UCI: 'go' */
	if (!strncmp(buffer,"go ",3))
		if (exploration_state) {
			exploration_state=0;
			strcpy(cmd_queue,buffer);
		} else {
			Iterate(g,buffer+3);
			fflush(stdout);
			return 0;
		}
	
	/* UCI: 'position' */
	if (!strncmp(buffer,"position ",9))
		if (exploration_state) {
			exploration_state=0;
			strcpy(cmd_queue,buffer);
		} else {
			s=buffer+9;
			if (!strncmp(s,"fen ",4))
				s=ParseFEN(g,s+4);
			else if (!strncmp(s,"startpos",8)) {
				s+=9;
				InitOriginalPosition(g);
			} else {
				Error(errDetails,"erroneous position statement!");
				return 0;
			}
			if (s && !strncmp(s,"moves ",6)) {
				s+=6;
				tok=strtok(s," ");
				do {
					if (move=ParseMove(g,tok)) {
						DoMove(g,move,&g->passed_backup[2*g->fullmoves_number+!Color-2]);
						if(Color==P_BLACK) g->fullmoves_number++;
						SwapColors;
						UpdateHalfmoveIndex;
					} else
						Error(errDetails,"erroneous move in 'position' command");
				} while (tok=strtok(NULL," "));
			}
			return 0;
		}
	return 0;
}
