
#include "const.h"
#include "types.h"
#include "globals.h"
#include "macros.h"
#include "proto.h"
#include "matrices.h"


/*********************************************************************************
* UNDOMOVE: Typically executed shortly after the use of DoMove() to restore the  *
* original situation. Nothing *very* exciting here, just the opposite of DoMove. *
*********************************************************************************/
void UnDoMove(GAME *g, SITBACKUP *backup) {
	/* update several 'check for draw' counters */
	g->sit.rule50counter=backup->rule50counter;
	g->sit.last_irreversible_move=backup->last_irreversible_move;
	
	/* restore en passant square and castling permissions */
	if(g->sit.enpassant)
		XORZobristEP(g->sit.enpassant);
	g->sit.enpassant=backup->enpassant;
	if(g->sit.enpassant)
		XORZobristEP(g->sit.enpassant);

	/* restore king's position */
	if (backup->src_figure==KING)
		g->sit.king[Color]=backup->src;
	
	/* restore castling rights */
	XORZobristCastling;
	g->sit.castling_disallowed[Color]=backup->castling_disallowed[Color];
	XORZobristCastling;

	/* restore special moves */
	switch(backup->move_type) {
		case CASTLING:
			if(Line(backup->dst)==6) {
				XOROwnSituationBit(ROOK,rooks,ColCmp(H1,H8));
				XOROwnSituationBit(ROOK,rooks,ColCmp(F1,F8));
				Piece(ColCmp(H1,H8))=ColCmp(ROOK,-ROOK);
				Piece(ColCmp(F1,F8))=0;
			} else if(Line(backup->dst)==2) {
				XOROwnSituationBit(ROOK,rooks,ColCmp(A1,A8));
				XOROwnSituationBit(ROOK,rooks,ColCmp(D1,D8));
				Piece(ColCmp(A1,A8))=ColCmp(ROOK,-ROOK);
				Piece(ColCmp(D1,D8))=0;
			} else 
				Error(errDetails,"Illegal Castling, backup->Line(dst) is neither 6 nor 2!");
			break;
		case ENPASSANT_CAPTURE:
			XOREnemySituationBit(PAWN,pawns,backup->dst+ColCmp(-8,8));
			Piece(backup->dst+ColCmp(-8,8))=ColCmp(-PAWN,PAWN);
			g->sit.material_score[!Color]+=V_PAWN;
			g->sit.pawn_count[!Color]+=1;
			break;
	}

	/* restore promoted pawns */
	if(backup->src_figure!=backup->dst_figure) {
		g->sit.material_score[Color]-=piece_value[backup->dst_figure]-V_PAWN;
		g->sit.pawn_count[Color]+=1;
	}

	/* remove recently set figure */
	switch(backup->dst_figure) {
		case PAWN:   XOROwnSituationBit(PAWN,pawns,backup->dst);     break;
		case KNIGHT: XOROwnSituationBit(KNIGHT,knights,backup->dst); break;
		case BISHOP: XOROwnSituationBit(BISHOP,bishops,backup->dst); break;
		case ROOK:   XOROwnSituationBit(ROOK,rooks,backup->dst);     break;
		case QUEEN:  XOROwnSituationBit(QUEEN,queens,backup->dst);   break;
		case KING:   XOROwnFigureset(backup->dst);                   
		             XORZobristOwnKing(backup->dst);                 break;
	}
	
	/* restore captured figure */
	if(backup->captured) {
		switch(backup->captured) {
			case PAWN:   XOREnemySituationBit(PAWN,pawns,backup->dst);     break;
			case KNIGHT: XOREnemySituationBit(KNIGHT,knights,backup->dst); break;
			case BISHOP: XOREnemySituationBit(BISHOP,bishops,backup->dst); break;
			case ROOK:   XOREnemySituationBit(ROOK,rooks,backup->dst);     break;
			case QUEEN:  XOREnemySituationBit(QUEEN,queens,backup->dst);   break;
			case KING:   Error(errDetails,"king capture detected!");
		}
		Piece(backup->dst)=ColCmp(-backup->captured,backup->captured);
		g->sit.material_score[!Color]+=piece_value[backup->captured];
		if(backup->captured==PAWN)
			g->sit.pawn_count[!Color]+=1;
	} else
		Piece(backup->dst)=0;
	
	/* restore old figure on original position */
	switch(backup->src_figure) {
		case PAWN:   XOROwnSituationBit(PAWN,pawns,backup->src);     break;
		case KNIGHT: XOROwnSituationBit(KNIGHT,knights,backup->src); break;
		case BISHOP: XOROwnSituationBit(BISHOP,bishops,backup->src); break;
		case ROOK:   XOROwnSituationBit(ROOK,rooks,backup->src);     break;
		case QUEEN:  XOROwnSituationBit(QUEEN,queens,backup->src);   break;
		case KING:   XOROwnFigureset(backup->src);                   
		             XORZobristOwnKing(backup->src);                 break;
	}
	
	UpdateHalfmoveIndex;
	/* restore old piece */
	Piece(backup->src)=ColCmp(backup->src_figure,-backup->src_figure);
}